<?php

namespace FiloBlu\ScratchAndWin\Model;

use Magento\Framework\Model\AbstractModel;

class ScratchAndWin extends AbstractModel {

    const CACHE_TAG = 'fb_scratchandwin';

    protected $_cacheTag = self::CACHE_TAG;

    protected $scratchAndWinCollectionFactory;
    protected $scratchAndWinDetailsCollectionFactory;
    protected $scratchAndWinTranslationsFactory;
    protected $scratchAndWinDetailsTranslationsFactory;
    protected $storeManager;
    private $objectFactory;

    public function __construct(        
        \Magento\Framework\Model\Context $context, 
        \Magento\Framework\Registry $registry,
        \FiloBlu\ScratchAndWin\Model\ResourceModel\ScratchAndWin\CollectionFactory $scratchandwinCollectionFactory,
        \FiloBlu\ScratchAndWin\Model\ResourceModel\ScratchAndWinDetails\CollectionFactory $scratchAndWinDetailsCollectionFactory,
        \FiloBlu\ScratchAndWin\Model\ScratchAndWinTranslationsFactory $scratchAndWinTranslationsFactory,
        \FiloBlu\ScratchAndWin\Model\ScratchAndWinDetailsTranslationsFactory $scratchAndWinDetailsTranslationsFactory,
        \Magento\Store\Model\StoreManagerInterface $storemanager,
        \Magento\Framework\DataObjectFactory $objectFactory,
        \Magento\Framework\Model\ResourceModel\AbstractResource $resource = null,
        \Magento\Framework\Data\Collection\AbstractDb $resourceCollection = null,
        array $data = []
    ) {
        $this->scratchAndWinCollectionFactory = $scratchandwinCollectionFactory;
        $this->scratchAndWinDetailsCollectionFactory = $scratchAndWinDetailsCollectionFactory;
        $this->scratchAndWinTranslationsFactory = $scratchAndWinTranslationsFactory;
        $this->scratchAndWinDetailsTranslationsFactory = $scratchAndWinDetailsTranslationsFactory;
        $this->storeManager = $storemanager;
        $this->objectFactory = $objectFactory;
        parent::__construct($context, $registry, $resource, $resourceCollection, $data);
    }

    /**
     * Initialize resource model
     *
     * @return void
     */
    protected function _construct() {
        $this->_init('FiloBlu\ScratchAndWin\Model\ResourceModel\ScratchAndWin');
    }

    public function getAvailableStatuses() {
        return array(
            0 => "Disabled",
            1 => "Enabled"
        );
    }

    public function getDetailsPosition() {
        if (!$this->getId()) {
            return array();
        }
        $array = $this->getData('details_position');
        if (is_null($array)) {
            $array = $this->getResource()->getDetailsPosition($this);
            $this->setData('details_position', $array);
        }

        return $array;
    }  

    public function getScratchAndWinByCode($code)
    {

        $collection = $this->scratchAndWinCollectionFactory->create();
        $collection->addFieldToFilter('enabled', ["eq" => 1]);
        $collection->addFieldToFilter('code', ["eq" => $code]);

        $scratchandwin = $collection->getFirstItem();

        $scratchandwinEntity = $this->objectFactory->create();

        if ($scratchandwin) {
            $scratchandwinTranslations = $this->scratchAndWinTranslationsFactory->create()->loadByParentAndStoreId($scratchandwin->getId(), $this->storeManager->getStore()->getId());
            if ($scratchandwinTranslations->getId()) {
                $translations = unserialize($scratchandwinTranslations->getSerializedValues());
                $scratchandwinEntity->setData(array_merge($scratchandwin->getData(), $translations));
            } else {
                $scratchandwinEntity->setData($scratchandwin->getData());
            }
            
            return $scratchandwinEntity;
        } else {
            return null;
        }
    }

    public function getScratchAndWinDetails($id) {
        
        $collection = $this->scratchAndWinDetailsCollectionFactory->create();
        $collection->addFieldToFilter('enabled', ["eq" => 1]);            
    
        $collection->getSelect()->joinInner(
            array('related' => $collection->getTable('filoblu_scratchandwin_relations')),
            'related.filoblu_scratchandwin_details_id=main_table.id AND related.filoblu_scratchandwin_id='.$id,
            array('filoblu_scratchandwin_id')
        );

        $collection->setOrder('position','ASC');        

        //$scratchandwindetailsEntity = $this->objectFactory->create();

        foreach($collection as $item) {
            $scratchandwindetailsTranslations = $this->scratchAndWinDetailsTranslationsFactory->create()->loadByParentAndStoreId($item->getId(), $this->storeManager->getStore()->getId());
            if ($scratchandwindetailsTranslations->getId()) {
                $translations = unserialize($scratchandwindetailsTranslations->getSerializedValues());
                $item->setData(array_merge($item->getData(), $translations));
            } 

            //$scratchandwindetailsEntity->setData($item->getData());
        }
        
        return $collection;
    }

    public function save()
    {
        $this->_cacheTag = self::CACHE_TAG . '_' . $this->getId();
        return parent::save();
    }

    public function afterLoad()
    {
        $this->_cacheTag = self::CACHE_TAG . '_' . $this->getId();
        return parent::_afterLoad();
    }

    public function beforeDelete()
    {
        $this->_cacheTag = self::CACHE_TAG . '_' . $this->getId();
        return parent::beforeDelete();
    }
}
