define([
    "ko",
    "jquery",
    "jquery/ui",
    "prototype",
    "filoblu_scratchandwin_lib",
    'FiloBlu_ScratchAndWin/js/model/current-date'
], function(ko, $, ui, prototype, wScratchPad, modelDate) {
    "use strict";

    $.widget('filoblu.scratchandwin', {
        options: {
            identifier: null,
            parentCode: null,
            container: null,
            promocode: null,
            shortText: null,
            longText: null,
            bgImage: null,
            overlayImage: null,
            cursor: null,
            storage_key: 'scratchedpad',
            cursorSize: 80,
            percentage: 50,
            expirationDay: null,
            expirationHour: null,
            ctaLink: null
        },

        _create: function() {

            var self = this;

            ko.computed(function() {
                self.checkDate(modelDate());
            }, this);

        },

        checkDate: function(date) {

            if (!date) {
                return;
            }

            var self = this;

            var currentDay = date.day;
            var currentHour = date.hour;

            if (currentDay == self.options.expirationDay && currentHour < self.options.expirationHour) {

                return self.initScratch(true);

            } else {

                if (currentDay < self.options.expirationDay) {

                    return self.showPlaceholder();

                } else {

                    return self.showScratched();

                }

            }

        },

        initScratch: function(init) {
            var self = this,
                show = false,
                promoTextGA;

            if (init) {

                // show placeholder
                $('.placeholder', self.element).fadeIn();
                $('.scratch-out-text', $(self.element).parent()).fadeIn();

                // check if isset cookie & init scratchPad
                if (!$.cookieStorage.get(self.options.storage_key) || $.cookieStorage.get(self.options.storage_key) != self.options.identifier) {

                    $(self.element).parent().addClass('active');

                    if (typeof window.scratchSlider != 'undefined') {
                        var index = $(self.element).parent().parent().attr('data-index');
                        scratchSlider.slideTo(index);
                    }

                    $(self.element).wScratchPad({
                        // the size of the eraser
                        size: self.options.cursorSize,
                        // the randomized scratch image   
                        bg: self.options.bgImage,
                        // give real-time updates
                        realtime: true,
                        // The overlay image
                        fg: self.options.overlayImage,
                        // The cursor (coin) image
                        'cursor': 'url(' + self.options.cursor + ') 5 5, default',

                        scratchMove: function(e, percent) {
                            // Show the plain-text promo code and call-to-action when the scratch area is 50% scratched
                            if ((percent > self.options.percentage) && (self.options.shortText != '') && !show) {

                                if (self.options.shortText) {
                                    $('.scratch-promo', self.element).append(self.options.shortText);
                                    promoTextGA = $('<div>').html(self.options.shortText).text();
                                } else {
                                    promoTextGA = self.options.identifier;
                                }

                                if (self.options.longText) {
                                    $('.scratch-promo-description', self.options.container).addClass('show').append(self.options.longText);
                                }

                                if (self.options.ctaLink) {
                                    $(self.element).wrap("<a href='" + self.options.ctaLink + "'></a>");
                                }

                                if (window.ga) {
                                    window.ga('send', {
                                        hitType: 'event',
                                        eventCategory: 'ScratchAndWin',
                                        eventAction: 'click',
                                        eventLabel: self.options.parentCode + ' - ' + promoTextGA
                                    });
                                }

                                window.dataLayer = window.dataLayer || [];
                                dataLayer.push({
                                    'event': 'scratchandwin',
                                    'scratchandwin': {
                                        'category': 'ScratchAndWin',
                                        'action': 'click',
                                        'label': self.options.parentCode + ' - ' + promoTextGA
                                    }
                                });

                                show = true;

                                $.cookieStorage.set(self.options.storage_key, self.options.identifier);
                            }
                        }

                    });

                } else {

                    // if daily scratchpad has been scratched show scratched image
                    $(self.element).parent().addClass('scratched');
                    $('.placeholder', self.element).remove();
                    $('.scratch-promo', self.element).append(self.options.shortText);
                    $('.placeholder-scratched', self.element).fadeIn();
                    $('.scratch-out-text', $(self.element).parent()).fadeIn();
                }

            }

        },

        showPlaceholder: function() {
            var self = this;

            // future scratchpad
            $(self.element).parent().addClass('to-scratch');
            $('.placeholder', self.element).fadeIn();
            $('.scratch-out-text', $(self.element).parent()).fadeIn();
        },

        showScratched: function() {
            var self = this;

            // past scratchpad
            $(self.element).parent().addClass('scratched');
            $('.placeholder-scratched', self.element).fadeIn();
            $('.scratch-promo', self.element).append(self.options.shortText);
            $('.scratch-out-text', $(self.element).parent()).fadeIn();
        }

    });

    return $.filoblu.scratchandwin;
});